﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace BlanketyBlank
{
    public partial class Form1 : Form
    {
        public Form1(string[] args)
        {
            InitializeComponent();

            //MessageBox.Show(string.Format("args length: {0}", args.Length));

            if (args.Length > 0)
            {
                try
                {
                    string zipFile = null;
                    string fileNameNoExt = "";

                    /*
                     * We started from an editor - ProcessArgs(args) has checked the supplied args
                     * and we are good to go since we are here.
                     */
                    fromEditor = true;

                    // Set the title and provide launch file path and name.
                    this.Text = string.Format("{0} - {1}", wndwTitle, args[0]);

                    /*
                     * Set the msgLabel to indicate the application will exit after they click the
                     * copy to clipboard button.
                     */
                    msgLabel.BackColor = Color.Red;
                    msgLabel.Text = "STARTED FROM A CODE EDITOR - APPLICATION WILL EXIT AFTER YOU CLICK ON THE COPY TO CLIPBOARD BUTTON BELOW";

                    /*
                     * Get any selected text and insert into the input code text box.
                     * Any code editor selected code has been copied to clipboard.
                     * Also create the box list from the selected text.
                     */
                    ProcessInputCode();

                    /*
                     * Locate the document's .Zip file so we can display the document's boxes.
                     */
                    zipFile = args[0].Remove(args[0].LastIndexOf(".")) + ".zip";

                    if (!File.Exists(zipFile))
                    {
                        fileNameNoExt = Path.GetFileNameWithoutExtension(args[0]).ToUpper();

                        // Check the curDir folder:
                        zipFile = Path.Combine(curDir, fileNameNoExt) + ".zip";

                        if (!File.Exists(zipFile))
                        {
                            MessageBox.Show(string.Format("  Could not locate Zip file:\n\n  {0}\n\n  Can not build document box list.", zipFile), "Blankety Blank: Zip file not found.", MessageBoxButtons.OK, MessageBoxIcon.Warning);

                            zipFile = null;
                        }
                    }

                    if (!string.IsNullOrEmpty(zipFile))
                    {
                        ProcessDocFile(zipFile);
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.ToString());
                }
            }
            else
            {
                // Set the Form startup as a stand alone app title.
                this.Text = wndwTitle;
            }

            // Setting focus to the code input area.
            inText.Select();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        /*
         * Get the document's zip file and build the document's box list.
         */
        private void javascriptToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string zipFile;

            zipFile = GetFileName(curDir);

            if (!String.IsNullOrEmpty(zipFile))
            {
                ProcessDocFile(zipFile);
            }
        }

        private void codeBoxSelBtn_Click(object sender, EventArgs e)
        {
            switch (codeBoxSelBtn.Text)
            {
                // Switching to "Select Single" per button text, once they click it it will
                // read "Select Multiple" so if/when they click it again it will switch to "Multiple".
                case ("Select Single"):
                    codeBoxSelBtn.Text = "Select Multiple";
                    codeBoxList.SelectionMode = SelectionMode.MultiSimple;
                    break;

                // Switching to "Select Multiple" per button text, once they click it it will
                // read "Select Single" so if/when they click it again it will switch to "Single".
                case ("Select Multiple"):
                    codeBoxSelBtn.Text = "Select Single";
                    codeBoxList.SelectionMode = SelectionMode.MultiExtended;
                    break;
            }
        }

        private void codeDelBoxListBtn_Click(object sender, EventArgs e)
        {
            codeBoxList.Items.Clear();
        }

        /*
         * This event handler for the 'Exclusion Boxes' button, inserts selected boxes from
         * the 'Box List' built from the supplied JavaScript code into the 'Exclusion Boxes'
         * list as long as those boxes are neither in the 'Exclusion Boxes' nor the
         * 'Inclusion Boxes' lists.
         * 
         * The list is then sorted and redisplayed.
        */
        private void codeExclBtn_Click(object sender, EventArgs e)
        {
            List<string> exclusionList = new List<string>();
            string box;

            // First set a local List to contain all boxes in the 'Exclusion Boxes' list.
            foreach (string item in codeExclBoxList.Items)
            {
                exclusionList.Add(item);
            }

            /*
             * Now add the selected boxes in codeBoxList to the codeExclBoxList
             * if not in there already. Also making sure the box is not already
             * in the docInclBoxList.
             */
            for (int index = 0; index < codeBoxList.SelectedIndices.Count; index++)
            {
                box = codeBoxList.Items[codeBoxList.SelectedIndices[index]].ToString();

                if (exclusionList.Contains(box))
                {
                    MessageBox.Show(string.Format("  {0} is already present in the\n  'Exclusion Boxes' list.", box), "codeExclBtn_Click(): Box already in list.", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else if (docInclBoxList.Items.Contains(box))
                {
                    MessageBox.Show(string.Format("  {0} is already present in the\n  'Inclusion Boxes' list.", box), "codeExclBtn_Click(): Box already in list.", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    exclusionList.Add(box);
                }
            }

            if (exclusionList.Count > 0)
            {
                // Sort the list:
                exclusionList.Sort(new BoxCompare());

                // Empty the codeExclBoxList:
                codeExclBoxList.Items.Clear();

                // Re-populate the codeExclBoxList from the sorted exclusionList:
                foreach (string item in exclusionList)
                {
                    codeExclBoxList.Items.Add(item);
                }
            }
        }

        // Delete selection(s).
        private void codeExclDelSelBtn_Click(object sender, EventArgs e)
        {
            for (int index = codeExclBoxList.SelectedIndices.Count - 1; index >= 0; index--)
            {
                codeExclBoxList.Items.RemoveAt(codeExclBoxList.SelectedIndices[index]);
            }
        }

        // Delete the whole list.
        private void codeDelExclListBtn_Click(object sender, EventArgs e)
        {
            codeExclBoxList.Items.Clear();
        }

        /*
         * User can toggle this "ON/OFF" - "ON" will clear the codeExclBoxList
         * when either the 'Single Blank' or the 'Multi Blank' buttons are clicked.
         * This will be most useful when this application is run as a stand-alone.
        */
        private void codeExclClearModeBtn_Click(object sender, EventArgs e)
        {
            switch (codeExclClearModeBtn.Text)
            {
                case ("OFF"):
                    codeExclClearModeBtn.Text = "ON";
                    break;

                case ("ON"):
                    codeExclClearModeBtn.Text = "OFF";
                    break;
            }
        }

        private void docBoxSelBtn_Click(object sender, EventArgs e)
        {
            switch (docBoxSelBtn.Text)
            {
                // Switching to "Select Single" per button text, once they click it it will
                // read "Select Multiple" so if/when they click it again it will switch to "Multiple".
                case ("Select Single"):
                    docBoxSelBtn.Text = "Select Multiple";
                    docBoxList.SelectionMode = SelectionMode.MultiSimple;
                    break;

                // Switching to "Select Multiple" per button text, once they click it it will
                // read "Select Single" so if/when they click it again it will switch to "Single".
                case ("Select Multiple"):
                    docBoxSelBtn.Text = "Select Single";
                    docBoxList.SelectionMode = SelectionMode.MultiExtended;
                    break;
            }
        }

        private void docDelBoxListBtn_Click(object sender, EventArgs e)
        {
            // Reset the label else it will look like this document has no boxes.
            docBoxListLbl.Text = "Box List";

            docBoxList.Items.Clear();
        }

        /*
         * This event handler for the 'Inclusion Boxes' button, inserts selected boxes from
         * the document 'Box List' [holds all the boxes contained on a document] into the
         * 'Inclusion Boxes' list as long as those boxes are neither in the 'Exclusion Boxes'
         * nor the 'Inclusion Boxes' lists already.
         * 
         * The list is then sorted and redisplayed.
        */
        private void docInclBtn_Click(object sender, EventArgs e)
        {
            List<string> inclusionList = new List<string>();
            string box;

            // First set a local List to contain all boxes in the 'Inclusion Boxes' list.
            foreach (string item in docInclBoxList.Items)
            {
                inclusionList.Add(item);
            }

            /*
             * Now add the selected boxes in docBoxList to the docInclBoxList
             * if not in there already. Also making sure the box is not already
             * in the codeExclBoxList.
             */
            for (int index = 0; index < docBoxList.SelectedIndices.Count; index++)
            {
                box = docBoxList.Items[docBoxList.SelectedIndices[index]].ToString();

                if (inclusionList.Contains(box))
                {
                    MessageBox.Show(string.Format("  {0} is already present in the\n  'Inclusion Boxes' list.", box), "docInclBtn_Click(): Box already in list.", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else if (codeExclBoxList.Items.Contains(box))
                {
                    MessageBox.Show(string.Format("  {0} is already present in the\n  'Exclusion Boxes' list.", box), "docInclBtn_Click(): Box already in list.", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    inclusionList.Add(box);
                }
            }

            if (inclusionList.Count > 0)
            {
                // Sort the list:
                inclusionList.Sort(new BoxCompare());

                // Empty the docInclBoxList:
                docInclBoxList.Items.Clear();

                // Re-populate the docInclBoxList from the sorted inclusionList:
                foreach (string item in inclusionList)
                {
                    docInclBoxList.Items.Add(item);
                }
            }
        }

        // Delete selection(s).
        private void docInclDelSelBtn_Click(object sender, EventArgs e)
        {
            for (int index = docInclBoxList.SelectedIndices.Count - 1; index >= 0; index--)
            {
                docInclBoxList.Items.RemoveAt(docInclBoxList.SelectedIndices[index]);
            }
        }

        // Delete the whole list.
        private void docDelInclListBtn_Click(object sender, EventArgs e)
        {
            docInclBoxList.Items.Clear();
        }

        /*
         * User can toggle this "ON/OFF" - "ON" will clear the docInclBoxList
         * when the 'Single Blank' button is clicked. This will be most
         * useful when this application is run as a stand-alone.
        */ 
        private void docInclClearModeBtn_Click(object sender, EventArgs e)
        {
            switch (docInclClearModeBtn.Text)
            {
                case ("OFF"):
                    docInclClearModeBtn.Text = "ON";
                    break;

                case ("ON"):
                    docInclClearModeBtn.Text = "OFF";
                    break;
            }
        }

        private void custClearBtn_Click(object sender, EventArgs e)
        {
            custBlankEntry.Clear();
        }

        /*
         * User can toggle this "ON/OFF" - "ON" will clear the custBlankEntry
         * text when the 'Custom Blank' button is clicked. This will be most
         * useful when this application is run as a stand-alone.
        */ 
        private void custClearModeBtn_Click(object sender, EventArgs e)
        {
            switch (custClearModeBtn.Text)
            {
                case ("OFF"):
                    custClearModeBtn.Text = "ON";
                    break;

                case ("ON"):
                    custClearModeBtn.Text = "OFF";
                    break;
            }
        }

        private void multiClearBtn_Click(object sender, EventArgs e)
        {
            groupSizeEntry.Clear();
        }

        /*
         * User can toggle this "ON/OFF" - "ON" will clear the groupSizeEntry
         * text field when the 'Multi Blank' button is clicked. This will be most
         * useful when this application is run as a stand-alone.
        */ 
        private void multiClearModeBtn_Click(object sender, EventArgs e)
        {
            switch (multiClearModeBtn.Text)
            {
                case ("OFF"):
                    multiClearModeBtn.Text = "ON";
                    break;

                case ("ON"):
                    multiClearModeBtn.Text = "OFF";
                    break;
            }
        }

        /*
         * This will be used when a user first copies to the clipboard.
         * Launching from Visual Studio Code first copies selected text
         * to the clipboard.
         */
        private void copyFromClipBtn_Click(object sender, EventArgs e)
        {
            ProcessInputCode();
        }

        /*
         * Performs single mode blanking and if the OFF/ON toggle buttons are
         * "ON" we will also delete the corresponding list(s).
         */
        private void singleBlnkBtn_Click(object sender, EventArgs e)
        {
            PrepareToBlank("single");

            if (codeExclClearModeBtn.Text == "ON")
            {
                codeExclBoxList.Items.Clear();
            }

            if (docInclClearModeBtn.Text == "ON")
            {
                docInclBoxList.Items.Clear();
            }
        }

        /*
         * Perform multiple mode blanking and if the OFF/ON toggle button for
         * the Code 'Exclusion Boxes' list ia "ON" we will delete the list.
         * Same thing for the 'Multi Blank' - 'Group Size' field, clear this field
         * if the toggle button is "ON".
         */
        private void multiBlnkBtn_Click(object sender, EventArgs e)
        {
            PrepareToBlank("multiple");

            if (codeExclClearModeBtn.Text == "ON")
            {
                codeExclBoxList.Items.Clear();
            }

            if (multiClearModeBtn.Text == "ON")
            {
                groupSizeEntry.Clear();
            }
        }

        /*
         * Perform custom mode blanking and if the OFF/ON toggle button
         * is "ON" we will also delete the corresponding condition.
         */
        private void custBlnkBtn_Click(object sender, EventArgs e)
        {
            PrepareToBlank("custom");

            if (custClearModeBtn.Text == "ON")
            {
                custBlankEntry.Text = "";
            }
        }

        private void revBlnkBtn_Click(object sender, EventArgs e)
        {
            ReverseBlank();
        }

        private void delCodeBtn_Click(object sender, EventArgs e)
        {
            // Clear inText, outText, codeBoxList, and codeExclBoxList.
            ClearInputOutputCodeAndLists();

            // Setting focus to the code input area.
            inText.Select();
        }

        private void codeBoxListBtn_Click(object sender, EventArgs e)
        {
            // Populate the codeBoxList from the inText RichTextBox content.
            CreateBoxList();
        }

        private void delAllListsBtn_Click(object sender, EventArgs e)
        {
            // Reset the label else it will look like this document has no boxes.
            docBoxListLbl.Text = "Box List";

            // Kill them all!
            codeBoxList.Items.Clear();
            codeExclBoxList.Items.Clear();
            docBoxList.Items.Clear();
            docInclBoxList.Items.Clear();
        }

        /*
         * Copies text from the Ouput Code area [outText] to the clipboard.
         * If no text has been selected it defaults to copying ALL text to
         * the clipboard. If started from the code editor the application
         * will then exit(0).
        */
        private void copyToClipBtn_Click(object sender, EventArgs e)
        {
            string output = "";

            if (outText.SelectedText.Length > 0)
            {
                output = outText.SelectedText;
            }
            else
            {
                /*
                 * Trimming the ending white space chars, currently there are
                 * no blank lines in our code [produced by the converter] so
                 * an ending blank line WOULD be good for now, but we will be
                 * coding more and more by hand so you will always get an extra
                 * blank line which would be a nuisance.
                 */
                output = outText.Text.TrimEnd() + "\n";
            }

            if (String.IsNullOrEmpty(output))
            {
                Clipboard.Clear();
            }
            else
            {
                // Clipboard.SetText first clears the Clipboard.
                Clipboard.SetText(output, TextDataFormat.Text);
            }

            if (fromEditor)
            {
                Application.Exit();
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            /*
             * We must copy the pre-blank and the reverse blank master script
             * files from the server storage location to the user's temp folder
             * on their computer, else we could not use them. This is the only
             * way to get this to work, I should've node!
             */
            string tmpPath = "";
            string fName = "";

            try
            {
                // Make sure we still have the master files!
                if (!File.Exists(preBlank))
                {
                    MessageBox.Show(string.Format("  Could not verify/locate the file:\n\n  {0}\n\n  Aborting application.", preBlank), "File Not Found", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    Application.Exit();
                }
                else if (!File.Exists(revBlank))
                {
                    MessageBox.Show(string.Format("  Could not verify/locate the file:\n\n  {0}\n\n  Aborting application.", revBlank), "File Not Found", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    Application.Exit();
                }

                tmpPath = Path.GetTempPath();

                // ********** Pre-Blank **********

                // Remove path from the file name.
                fName = Path.GetFileName(preBlank);

                preBlankTmp = Path.Combine(tmpPath, fName);
                preBlankCode = Path.Combine(tmpPath, preBlankCodeName);

                // Overwrite the file if it exists.
                File.Copy(preBlank, preBlankTmp, true);

                // The master file is readonly, this allows us to delete
                // the user's copy.
                File.SetAttributes(preBlankTmp, FileAttributes.Normal);

                //MessageBox.Show(string.Format("tmpPath: {0}", tmpPath));

                // ********** Reverse Blank **********

                // Same pre-blank comments apply:
                fName = Path.GetFileName(revBlank);
                revBlankTmp = Path.Combine(tmpPath, fName);
                revBlankCode = Path.Combine(tmpPath, revBlankCodeName);
                File.Copy(revBlank, revBlankTmp, true);
                File.SetAttributes(revBlankTmp, FileAttributes.Normal);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            /*
             * Delete the files that were copied to the user's temp folder:
             */
            try
            {
                if (!String.IsNullOrEmpty(preBlankTmp))
                {
                    if (File.Exists(preBlankTmp))
                    {
                        File.Delete(preBlankTmp);
                    }
                }

                if (!String.IsNullOrEmpty(preBlankCode))
                {
                    if (File.Exists(preBlankCode))
                    {
                        File.Delete(preBlankCode);
                    }
                }

                if (!String.IsNullOrEmpty(revBlankTmp))
                {
                    if (File.Exists(revBlankTmp))
                    {
                        File.Delete(revBlankTmp);
                    }
                }

                if (!String.IsNullOrEmpty(revBlankCode))
                {
                    if (File.Exists(revBlankCode))
                    {
                        File.Delete(revBlankCode);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }
    }
}
